/*****************************************************

(sondelog_ptu.c)
This program read GPS sonde log data.

1.1 add the program of specific humidity(02.12.6)


                    02.11.27 made by Munehisa YAMAMOTO
*****************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define TIME 10800  //3h*3600s

int main(int argc, char *argv[])
{

    FILE	*fpi, *fpo;
    int		i, lagi, datanum;
    float	minRH, omomi;
    char	buf[256], bufmin[4], bufs[3], bufP[10];
    char	bufH[10], bufT[11], bufRH[7], bufDewp[6];
    int		min, s;
    int		t[TIME], H[TIME];
    float	P[TIME], T[TIME], RH[TIME], Dewp[TIME], tau[TIME], U[TIME];
    float	PT[TIME], EPT[TIME],q[TIME];

    float	PotentialTemp(float T, float P);
    float	EquivalentPotentialTemp(float T, float P, float U);
    float	SpecificHumidity(float T, float P, float U);
    float	TimelagRH(float T, float P);

    for(i=0;i<TIME;i++){
    	t[i]=0; H[i]=-9999; P[i]=-9999.; T[i]=-9999.; RH[i]=-9999.;
	RH[i]=-9999.; Dewp[i]=-9999.; PT[i]=-9999.; EPT[i]=-9999.;
	tau[i]=-9999.; q[i]=-9999.;
    }

    if(argc!=2){
    	printf("Usage: sondelog_ptu filename\n");
	exit(0);
    }

/*** Open File***/
    if((fpi=fopen(argv[1],"r"))==NULL){
    	printf("***inputfile cannot open***\n");
	exit(0);
    }
    fpo=fopen("test.csv","w");
    fprintf(fpo,"t(s),P(hPa),H(gpm),T(degC),PT(K),EPT(K),q(g/kg),U(%)\n");

/*** Read index***/
    for(i=0;i<5;i++) fgets(buf, 256, fpi);
    fgets(buf, 256, fpi); printf("%s",buf);  //StartTime
    for(i=0;i<2;i++) fgets(buf, 256, fpi);
    fgets(buf, 256, fpi); printf("%s",buf);  //Location
    for(i=0;i<2;i++) fgets(buf, 256, fpi);
    fgets(buf, 256, fpi); printf("%s",buf);  //RSnumber
    for(i=0;i<27;i++) fgets(buf, 256, fpi);

/*** Read data***/
    i=0; minRH=999;

    while(!feof(fpi)){
    	if(i>=TIME) break;

    	fscanf(fpi,"%4s%3s%9s%9s%10s%7s%8s",
		bufmin, bufs, bufP, bufH, bufT, bufRH, bufDewp);
	min = atoi(bufmin);
	  s = atoi(bufs);
	t[i] = s+min*60;
	P[i] = atof(bufP);
	H[i] = atoi(bufH);
	T[i] = atof(bufT);
	RH[i] = atof(bufRH);    if(RH[i]<minRH) minRH=RH[i];
	Dewp[i] = atoi(bufDewp);
	
	if(P[i]==0. && T[i]==0. && RH[i]==0.){
		P[i]=-9999.; T[i]=-9999.; RH[i]=-9999.;
	}

	i++;
    }
    datanum=i;

    if(minRH<0. && minRH>-5.){
    	for(i=0;i<datanum;i++)RH[i]-=minRH;
    }
    
/***calclation***/

    for(i=0;i<datanum;i++){
 
 /**calibrate RH time lag**/
	tau[i]=TimelagRH(T[i],P[i]);
	lagi=(int)((t[i]+tau[i])/2.);
	if(lagi<datanum){
		omomi=(t[i]+tau[i])/2.-lagi;
		U[i]=RH[lagi]*omomi+RH[lagi+1]*(1-omomi);
	}
	else U[i]=-9999.;
	if(U[i]<0.)U[i]=-9999.;


	PT[i]=PotentialTemp(T[i], P[i]);
	EPT[i]=EquivalentPotentialTemp(T[i], P[i], U[i]);
	q[i]=SpecificHumidity(T[i], P[i], U[i]);


	fprintf(fpo,"%d,%5.1f,%d,%3.1f,%3.1f,%3.1f,%3.2f,%3.1f\n",
		t[i], P[i], H[i], T[i], PT[i], EPT[i], q[i], U[i]);
    }

    fclose(fpi);
    fclose(fpo);
    return 0;
}

float PotentialTemp(float T, float P)
{
    float	R=0.28705; //(J g^-1 K^-1)
    float	Cp=1.005;  //(J g^-1 K^-1)
    float	PT;        //Potential Temperature(K);
    
    if(T<-273. || P<0.) PT=-9999.;
    else PT=(273.15+T)*pow((1000./P),(R/Cp));
    return PT;
}

float EquivalentPotentialTemp(float T, float P, float U)
{
    float	PotnetialTemp(), PT;  //Potential Temperature(K)
    float	L=2501.;  //(J g^-1)
    float	r, e, es;
    float	a=7.5, b=237.3;
    float	Cpd=1.005;  //(J g^-1 K^-1)
    float	EPT;  //Equivalent Potential Temperature(K)

    PT=PotentialTemp(T, P);
    es=6.11*pow(10.,(a*T)/(b+T));
    e=es*(U/100.);
    r=0.62198*e/P;
    if(PT<0 || U<0.) EPT=-9999.;
    else EPT=PT*exp((L*r)/(Cpd*(T+273.15)));
    return EPT;
}

float SpecificHumidity(float T, float P, float U)
{
    float	r;  //mixing ratio
    float	q;  //specific humidity(g/kg)
    float	e;  //water vapor pressure(hPa)
    float	es; //saturated water vapor pressure(hPa)
    float	a=7.5, b=237.3;


    es=6.11*pow(10.,(a*T)/(b+T));
    e=es*(U/100.);
    r=0.62198*e/(P-e);
    if(T<-273. || P<0. || U<0.)q=-9999.;
    else q=1000.*r/(1.+r);
    return q;
}

float TimelagRH(float T, float P)
{
    float	tauT;  //thermal time lag
    float	tauH;  //humidity time lag
    float	tau0=0.65; //time lag of HUMICAP (s)
    float	p0=1000.;  //standard pressure(hPa)
    float	u0=5.;  //standard through wind velocity (m/s)
    float	u=5.;  //actual through wind velocity
    float	tau;  //time lag
    
    tauT=tau0*pow(p0/P,0.5)*pow(u0/u,0.5);
    tauH=2.085*pow(10.,(-0.0331*T));
    if(T<-273. || P<0.)tau=0.;
    else tau=tauT+tauH;
    return tau;
}
